﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Design;
using System.IO;
using GrapeCity.ActiveReports.Design;
using GrapeCity.ActiveReports.Design.Resources;
using GrapeCity.ActiveReports;
using System.Reflection;
using System.Runtime.InteropServices;

namespace GrapeCity.ActiveReports.Samples.EndUserDesigner
{
    public partial class UnifiedDesignerForm : Form
    {
        [DllImport("user32.dll", CharSet = CharSet.Auto, CallingConvention = CallingConvention.StdCall)]
        public static extern void mouse_event(long dwFlags, long dx, long dy, long cButtons, long dwExtraInfo);

        private const int MOUSEEVENTF_LEFTDOWN = 0x02;
        private const int MOUSEEVENTF_LEFTUP = 0x04;
        private const int MOUSEEVENTF_RIGHTDOWN = 0x08;
        private const int MOUSEEVENTF_RIGHTUP = 0x10;

        public void DoMouseClick()
        {
            //Call the imported function with the cursor's current position
            int X = Cursor.Position.X;
            int Y = Cursor.Position.Y;
            mouse_event(MOUSEEVENTF_LEFTDOWN | MOUSEEVENTF_LEFTUP, X, Y, 0, 0);
        }

        ToolStripDropDownItem filemenu;
        public UnifiedDesignerForm()
        {

            InitializeComponent();
            //Populating the ToolBox,reportExplorer and Propertygrid
            this.reportdesigner.Toolbox = this.reporttoolbox;//Attaches the toolbox to the report designer
            this.reportExplorer.ReportDesigner = this.reportdesigner;//Attaches the report explorer to the report designer
            this.reportdesigner.PropertyGrid = this.reportpropertyGrid;//Attaches the Property Grid to the report designer
            //Populating the Menu
            ToolStrip toolstrip = this.reportdesigner.CreateToolStrips(new DesignerToolStrips[]{
               DesignerToolStrips.Menu,
            })[0];
            filemenu = (ToolStripDropDownItem)toolstrip.Items[0];
            this.CreateFileMenu(filemenu);

            this.AppendToolStrips(0, new ToolStrip[]
                    {
                            toolstrip
                     });
            this.AppendToolStrips(1, this.reportdesigner.CreateToolStrips(new DesignerToolStrips[]
                    {
                         DesignerToolStrips.Edit, 
                         DesignerToolStrips.Undo, 
                         DesignerToolStrips.Zoom
                        
                     }));
            ToolStrip item = this.CreateReportToolbar();
            this.AppendToolStrips(1, new List<ToolStrip>
	             {
		               item
	             });
            this.AppendToolStrips(2, this.reportdesigner.CreateToolStrips(new DesignerToolStrips[]
                {
                   DesignerToolStrips.Format, 
                   DesignerToolStrips.Layout
                }));

            LoadTools(this.reporttoolbox);
        }
        private static void LoadTools(IToolboxService toolbox)
        {
            //Add Data Providers
            foreach (Type type in new Type[]
				{
					typeof (System.Data.DataSet),
					typeof (System.Data.DataView),
					typeof (System.Data.OleDb.OleDbConnection),
					typeof (System.Data.OleDb.OleDbDataAdapter),
					typeof (System.Data.Odbc.OdbcConnection),
					typeof (System.Data.Odbc.OdbcDataAdapter),
					typeof (System.Data.SqlClient.SqlConnection),
					typeof (System.Data.SqlClient.SqlDataAdapter)
				})
            {
                toolbox.AddToolboxItem(new ToolboxItem(type), "Data");
            }
        }
        //Adding DropDownItems to the ToolStripDropDownItem
        private void CreateFileMenu(ToolStripDropDownItem fileMenu)
        {
            fileMenu.DropDownItems.Clear();
            fileMenu.DropDownItems.Add(new ToolStripMenuItem("New", Images.CmdNewReport, new EventHandler(this.OnNew), (Keys)131150));
            fileMenu.DropDownItems.Add(new ToolStripMenuItem("Open", Images.CmdOpen, new EventHandler(this.OnOpen), (Keys)131151));
            fileMenu.DropDownItems.Add(new ToolStripMenuItem("Export", null, new EventHandler(this.OnExport), (Keys)131152));
            fileMenu.DropDownItems.Add(new ToolStripMenuItem("Save", Images.CmdSave, new EventHandler(this.OnSave), (Keys)131155));
            fileMenu.DropDownItems.Add(new ToolStripMenuItem("Save As", Images.CmdSaveAs, new EventHandler(this.OnSaveAs)));
            fileMenu.DropDownItems.Add(new ToolStripSeparator());
            fileMenu.DropDownItems.Add(new ToolStripButton("Exit", null, new EventHandler(this.OnExit)));

        }

        private ToolStrip CreateReportToolbar()
        {
            return new ToolStrip(new ToolStripButton[]
            {
                UnifiedDesignerForm.CreateToolStripButton("New",Images.CmdNewReport,new EventHandler(this.OnNew),"New"),
                UnifiedDesignerForm.CreateToolStripButton("New",Images.CmdOpen,new EventHandler(this.OnOpen),"New"),
                UnifiedDesignerForm.CreateToolStripButton("Save",Images.CmdSave,new EventHandler(this.OnSave),"Save")
            });
        }
        //Getting the Designer to open a new Report on Menu Item "New" click
        private void OnNew(object sender, EventArgs e)
        {
            if (this.ConfirmSaveChanges())
            {
                this.reportdesigner.ExecuteAction(DesignerAction.NewReport);
            }
        }
        //Getting the Designer to open a Report on Menu Item "Open" click
        private void OnOpen(object sender, EventArgs e)
        {
            if (this.ConfirmSaveChanges())
            {
                this.reportdesigner.ExecuteAction(DesignerAction.FileOpen);
            }
        }

        private void OnExport(object sender, EventArgs e)
        {

            try
            {
                GrapeCity.ActiveReports.SectionReport _report = new GrapeCity.ActiveReports.SectionReport();
                _report = (GrapeCity.ActiveReports.SectionReport)this.reportdesigner.Report;
                _report.Run();
                ExportProperties _export = new ExportProperties(_report.Document);
                _export.Show();
            }
            catch
            {
                MessageBox.Show("There was an error in processing the report.Please redesign the same.");
            }


        }
        //Getting the Designer to Save the Report on Menu Item "Save" click
        private void OnSave(object sender, EventArgs e)
        {
            this.reportdesigner.ExecuteAction(DesignerAction.FileSave);
        }
        //Getting the Designer to Save the Report on Menu Item "Save As" click
        private void OnSaveAs(object sender, EventArgs e)
        {

            this.reportdesigner.ExecuteAction(DesignerAction.FileSave);
        }
        private void OnExit(object sender, EventArgs e)
        {
            base.Close();
        }
        //Checking whether modifications have been made to the report loaded to the designer
        private bool ConfirmSaveChanges()
        {
            if (this.reportdesigner.IsDirty)
            {
                DialogResult dialogresult = MessageBox.Show("Report has been changed!!Do you wish to save it?", "", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question);
                if (dialogresult == DialogResult.Cancel)
                {
                    return false;
                }
                else if (dialogresult == DialogResult.Yes)
                {
                    this.reportdesigner.ExecuteAction(DesignerAction.FileSave);
                }

            }

            return true;

        }

        private void AppendToolStrips(int row, IList<ToolStrip> toolStrips)
        {
            ToolStripPanel topToolStripPanel = this.toolStripContainer1.TopToolStripPanel;
            int num = toolStrips.Count;
            while (--num >= 0)
            {
                topToolStripPanel.Join(toolStrips[num], row);
            }
        }

        private static ToolStripButton CreateToolStripButton(string text, Image image, EventHandler handler, string toolTip)
        {
            return new ToolStripButton(text, image, handler)
            {
                DisplayStyle = ToolStripItemDisplayStyle.Image,
                ToolTipText = toolTip,
                DoubleClickEnabled = true
            };
        }

        TreeView rETree = new TreeView();
        private void reportExplorer_Load(object sender, EventArgs e)
        {
            rETree = (TreeView)reportExplorer.Controls[0].Controls[0];
            rETree.HideSelection = false;
            rETree.NodeMouseClick += new TreeNodeMouseClickEventHandler(rETree_NodeMouseClick);
            rETree.KeyDown += new KeyEventHandler(rETree_KeyDown);
        }

        void rETree_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                MessageBox.Show("You are not allowed to delete it");
            }
        }

        void rETree_NodeMouseClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            if (e.Button == System.Windows.Forms.MouseButtons.Right)
            {
                if (e.Node.Level >= 0)
                {
                    Point p = Cursor.Position;
                    int screenwidth = Screen.PrimaryScreen.Bounds.Width;
                    int leftWidth = screenwidth - p.X;
                    if (leftWidth < 210)
                        p.Y -= 5;
                    else
                        p.X -= 10;
                    Cursor.Position = p;
                    DoMouseClick();
                    if (leftWidth < 210)
                        p.Y += 5;
                    else
                        p.X += 10;
                    Cursor.Position = p;
                    DoMouseClick();
                }

            }
        }




    }
}

